var through = require('through2');
var fs = require('fs');
var path = require('path');
var gutil = require('gulp-util');
var PluginError = gutil.PluginError;
var iconv = require('iconv-lite');
var spawn = require('child_process').spawn;
var helper = require('./assets/helper');
var constants = require('./assets/constants');

module.exports = function(srcDir, opt){
  if (!opt) opt = {};

  if (!srcDir) throw new PluginError(constants.PLUGIN_NAME, 'STYLE_BUILD_ERR: Missing srcDir option for ' + constants.PLUGIN_NAME);

  function transformFunc(file, encoding, transCallback) {
    var that = this;
    if (file.isNull()) return; // ignore
    if (file.isStream()) return this.emit('error', new PluginError(constants.PLUGIN_NAME,  'STYLE_BUILD_ERR: Streaming not supported'));
    
    var fileContent = iconv.decode(file.contents, 'GBK');
    var spawnCmdList = ['-jar', __dirname + '/lib/yuicompressor-2.4.7.jar', '--charset', 'gbk'];

    // 是否有压缩标识
    if (constants.COMPRESS_FLAG_RE.test(fileContent)) {
      // 提取压缩种类字符串
      if (constants.COMPRESS_OPT.test(fileContent)) {
        spawnCmdList.push(constants.COMPRESS_OPT.exec(fileContent)[1])
      }
      spawnCmdList.push(file.path);
      var comp = spawn('java', spawnCmdList);
      var bufList = [];
      var errorInfo = '';
      comp.stdout.on('data', function(data) {
        bufList.push(data);
      });
      comp.stderr.on('data', function(data) {
        errorInfo += data;
      });
      comp.on('close', function() {
        if( errorInfo ) {
          throw new Error('STYLE_BUILD_ERR: ' + errorInfo);
        }
        // 是否需要中文转unicode
        if (constants.CONV_TO_UNICODE_RE.test(fileContent)) {
          var unicodeFileCont = helper.toUnicode(iconv.decode(Buffer.concat(bufList), 'GBK'));
          file.contents = iconv.encode(unicodeFileCont, 'GBK');
        } else {
          file.contents = Buffer.concat(bufList);
        }
        that.push(file);
        transCallback();
      });
    } else {
      file.contents = iconv.encode(fileContent, 'GBK');
      this.push(file);
      transCallback();
    }
  }

  return through.obj(transformFunc);
};