require('chai').should();
require('mocha');

var fs = require('fs');
var path = require('path');
var constants = require('../assets/constants');
var helper = require('../assets/helper');
var gutil = require('gulp-util');
var iconv = require('iconv-lite');
var PluginError = gutil.PluginError;

describe('regular expression testing', function() {
	it('a compress flag str exists', function() {
		var str = '/*cmd:compress=true*/';
		var str2 = '/*!!cmd:compress=true*/';
		constants.COMPRESS_FLAG_RE.test(str).should.be.true;
		constants.COMPRESS_FLAG_RE.test(str2).should.be.true;
	})

	it('a convert to unicode flag str exists', function() {
		var str = '/*cmd:conv2unicode=true*/';
		var str2 = '/*!!cmd:conv2unicode=true*/';
		constants.CONV_TO_UNICODE_RE.test(str).should.be.true;
		constants.CONV_TO_UNICODE_RE.test(str2).should.be.true;
	})

	it('a compress type str exists', function() {
		var str = '/*cmd:compress=true*/ \n'
			+ '/*cmd:jsCompressOpt=["--disable-optimizations"]*/';
		var str2 = '/*cmd:compress=true*/ \n'
			+ '/*cmd:jsCompressOpt=["--nomunge"]*/';
		constants.COMPRESS_OPT.test(str).should.be.true;
		constants.COMPRESS_OPT.test(str2).should.be.true;
		constants.COMPRESS_OPT.exec(str)[1].should.equal("--disable-optimizations");
		constants.COMPRESS_OPT.exec(str2)[1].should.equal("--nomunge");
	})
})

describe('helper testing', function() {
	it('test toUnicode func', function() {
		var unicode = helper.toUnicode('中文');
		unicode.should.equal('\\u4e2d\\u6587');
	})

	it('test toUnicode func - from actual gbk file', function() {
		var buf = fs.readFileSync(path.join(__dirname, 'style_test/chinese.txt'));
		var utf8Str = iconv.decode(buf, 'GBK');
		var unicode = helper.toUnicode(utf8Str);
		unicode.should.equal('\\u4e2d\\u6587');
	})
})

describe('error testing', function() {
	it('srcDir must not be missing', function() {
		require('../index').should.throw(Error); 
	})
})